
#include <stdio.h>
#include <pthread.h>
#include <mfx/mfxvideo.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <va/va.h>
#include <va/va_drm.h>
#include <va/va_x11.h>
#include <X11/Xlib.h>
#include <string.h>
#include <stdlib.h>

int m_fd = -1;
mfxBitstream mfxBS;
mfxSession session = NULL;
int running = 1;
mfxVideoParam mfxVideoParams;

static void *__dec_hdr(void *p)
{
    FILE *fp = NULL;
    int len = 0;
    mfxStatus sts = MFX_ERR_NONE;
    
    fp = fopen("./2.264", "r");
    if (!fp) {
        printf("open file ./2.264 error!\n");
        goto out;
    }

    do {
        memmove(mfxBS.Data, mfxBS.Data + mfxBS.DataOffset, mfxBS.DataLength);
        mfxBS.DataOffset = 0;
        len = fread(mfxBS.Data + mfxBS.DataLength, 1, mfxBS.MaxLength - mfxBS.DataLength, fp);
        mfxBS.DataLength += len;
        sts = MFXVideoDECODE_DecodeHeader(session, &mfxBS, &mfxVideoParams);
    } while (sts != MFX_ERR_NONE);

    printf("\nmfx decode header success!!!\n\n");

out:
    running = 0;
    return NULL;
}

VADisplay init_va_display(void)
{
    VADisplay va_dpy = NULL;
    int va_ver_major, va_ver_minor, err;
    int i;
    char *dri[] = {
        "/dev/dri/card0",
        "/dev/dri/renderD128",
    };

    for (i=0; i<sizeof(dri)/sizeof(dri[0]); i++) {
        m_fd = open(dri[i], O_RDWR);
        if (m_fd < 0) {
            printf("open %s error!!!!\n", dri[i]);
            continue;
        }
        printf("open %s success!!!!\n", dri[i]);
        va_dpy = vaGetDisplayDRM(m_fd);
        err = vaInitialize(va_dpy, &va_ver_major, &va_ver_minor);
        if (err != VA_STATUS_SUCCESS) {
            printf("Cannot initialize VA: %s\n", vaErrorStr(err));
            close(m_fd);
            continue;
        }
        printf("Initialized VA v%d.%d\n", va_ver_major, va_ver_minor);
        break;
    }

    return va_dpy;
}

int usage(void)
{
    printf("./demo [stack_size]\n");
    return 0;
}

int main(int argc, char **argv)
{
    mfxInitParam par = {0};
    VADisplay va_dpy = NULL;
    mfxVersion version = {{1, 1}};
    mfxStatus sts = MFX_ERR_NONE;
    pthread_t thread;
    pthread_attr_t attr;
    char *desc = NULL;
    mfxIMPL mfx_impl = {0};
    int stack_size = 1024*16;
    
    if (argc <= 2) {
        if (argc == 2) {
            int ss = atoi(argv[1]);
            
            stack_size = stack_size > ss ? stack_size : ss;
        }
    } else {
        usage();
        goto ERR;
    }
    
    memset(&attr, 0, sizeof(pthread_attr_t));
    memset(&mfxVideoParams, 0, sizeof(mfxVideoParam));
    memset(&mfxBS, 0, sizeof(mfxBitstream));
    
    mfxVideoParams.mfx.CodecId = MFX_CODEC_AVC;
    mfxVideoParams.IOPattern = MFX_IOPATTERN_OUT_VIDEO_MEMORY;
    
    mfxBS.MaxLength = 1024 * 16;
    mfxBS.DataLength = 0;
    mfxBS.Data = malloc(mfxBS.MaxLength);
    if (!mfxBS.Data) {
        printf("bitstream data alloc memory error!\n");
        goto ERR;
    }

    printf("use stack size is %d\n", stack_size);

    va_dpy = init_va_display();
    if (!va_dpy) {
        printf("va display init error!\n");
        goto ERR;
    }

    par.Implementation = MFX_IMPL_HARDWARE_ANY;
    par.Version.Major = 1;
    par.Version.Minor = 17;
    
    sts = MFXInitEx(par, &session);
    if (sts != MFX_ERR_NONE) {
        printf("MFX Init session error : %d\n", sts);
        goto ERR;
    }
    
    sts = MFXQueryVersion(session , &version); // get real API version of the loaded library
    if (sts != MFX_ERR_NONE) {
        goto ERR;
    }
    
    MFXQueryIMPL(session, &mfx_impl);
    switch (MFX_IMPL_BASETYPE(mfx_impl)) {
        case MFX_IMPL_SOFTWARE:
            desc = "software";
            break;
        case MFX_IMPL_HARDWARE:
        case MFX_IMPL_HARDWARE2:
        case MFX_IMPL_HARDWARE3:
        case MFX_IMPL_HARDWARE4:
            desc = "hardware accelerated";
            break;
        default:
            desc = "unknown";
    }

    printf("impl desc is [%s] version is %d.%d\n", desc, version.Major, version.Minor);
    
    pthread_attr_setstacksize(&attr, stack_size);
    pthread_create(&thread, &attr, __dec_hdr, NULL);
    
    while (running) {
        sleep(1);
    }

ERR:
    if (mfxBS.Data) {
        free(mfxBS.Data);
    }
    if (va_dpy) {
        vaTerminate(va_dpy);
    }
    if (m_fd > 0) {
        close(m_fd);
    }
    return 0;
}

